/*
 * (c) 2012 Jens Mueller
 *
 * Kleincomputer-Emulator
 *
 * Komponente fuer die VCS80-Tastatur
 */

package jkcemu.emusys.etc;

import java.awt.*;
import java.lang.*;
import java.util.Arrays;
import jkcemu.Main;
import jkcemu.base.*;
import jkcemu.emusys.VCS80;


public class VCS80KeyboardFld extends AbstractKeyboardFld
{
  private static final int MARGIN    = 10;
  private static final int FONT_SIZE = 18;
  private static final int KEY_SIZE  = 40;

  private VCS80 vcs80;
  private Font  fontBtn;
  private int[] kbMatrix;
  private int   curIdx;
  private int   curX;
  private int   curY;


  public VCS80KeyboardFld( VCS80 vcs80 )
  {
    super( 24 );
    this.vcs80    = vcs80;
    this.fontBtn  = new Font( "SansSerif", Font.PLAIN, FONT_SIZE );
    this.kbMatrix = new int[ 8 ];
    this.curIdx   = 0;
    this.curX     = MARGIN;
    this.curY     = MARGIN;
    addKey( "C", 3, 0x20 );
    addKey( "D", 2, 0x20 );
    addKey( "E", 1, 0x20 );
    addKey( "F", 0, 0x20 );
    this.curX += (KEY_SIZE / 2);
    addKey( "PE", 0, 0x40 );
    addKey( "ST", 1, 0x40 );

    this.curX = MARGIN;
    this.curY += KEY_SIZE;
    addKey( "8", 7, 0x20 );
    addKey( "9", 6, 0x20 );
    addKey( "A", 5, 0x20 );
    addKey( "B", 4, 0x20 );
    this.curX += (KEY_SIZE / 2);
    addKey( "TR", 2, 0x40 );
    addKey( "GO", 3, 0x40 );

    this.curX = MARGIN;
    this.curY += KEY_SIZE;
    addKey( "4", 3, 0x10 );
    addKey( "5", 2, 0x10 );
    addKey( "6", 1, 0x10 );
    addKey( "7", 0, 0x10 );
    this.curX += (KEY_SIZE / 2);
    addKey( "RE", 4, 0x40 );
    addKey( "MA", 5, 0x40 );

    this.curX = MARGIN;
    this.curY += KEY_SIZE;
    addKey( "0", 7, 0x10 );
    addKey( "1", 6, 0x10 );
    addKey( "2", 5, 0x10 );
    addKey( "3", 4, 0x10 );
    this.curX += (KEY_SIZE / 2);
    addKey( "A-", 6, 0x40 );
    addKey( "A+", 7, 0x40 );

    int h = this.curY + KEY_SIZE + MARGIN;
    setPreferredSize(
	new Dimension(
		(2 * MARGIN) + (6 * KEY_SIZE) + (KEY_SIZE / 2),
		(2 * MARGIN) + (4 * KEY_SIZE) ) );
  }


	/* --- ueberschriebene Methoden --- */

  @Override
  public boolean accepts( EmuSys emuSys )
  {
    return emuSys instanceof VCS80;
  }


  @Override
  public String getKeyboardName()
  {
    return "VCS80-Tastatur";
  }


  @Override
  protected void keySelectionChanged()
  {
    Arrays.fill( this.kbMatrix, 0 );
    synchronized( this.selectedKeys ) {
      for( KeyData key : this.selectedKeys ) {
	if( (key.col >= 0) && (key.col < this.kbMatrix.length) ) {
	  this.kbMatrix[ key.col ] |= key.value;
	}
      }
    }
    this.vcs80.updKeyboardMatrix( this.kbMatrix );
  }


  @Override
  protected void paintComponent( Graphics g )
  {
    g.setFont( this.fontBtn );;
    g.setPaintMode();
    g.setColor( Color.lightGray );
    g.fillRect( 0, 0, getWidth(), getHeight() );
    for( KeyData key : this.keys ) {
      boolean selected = isKeySelected( key );
      if( selected ) {
	g.setColor( Color.gray );
	g.fillRect( key.x + 1, key.y + 1, key.w - 1, key.h - 1 );
      }
      g.setColor( Color.lightGray );
      g.draw3DRect( key.x + 1, key.y + 1, key.w - 1, key.h - 1, !selected );
      if( key.text1 != null ) {
	FontMetrics fm = g.getFontMetrics();
	if( fm != null ) {
	  g.setColor( Color.black );
	  g.drawString(
		key.text1,
		key.x + ((key.w - fm.stringWidth( key.text1 )) / 2) + 1,
		key.y + FONT_SIZE + ((key.h - FONT_SIZE) / 2) - 1 );
	}
      }
    }
  }


  @Override
  public void setEmuSys( EmuSys emuSys )
  {
    if( emuSys instanceof VCS80 ) {
      this.vcs80 = (VCS80) emuSys;
    } else {
      throw new IllegalArgumentException( "EmuSys != VCS80" );
    }
  }


	/* --- private Methoden --- */

  private void addKey( String text, int col, int value )
  {
    this.keys[ this.curIdx++ ] = new KeyData(
					this.curX,
					this.curY,
					KEY_SIZE,
					KEY_SIZE,
					text,
					null,
					null,
					null,
					null,
					col,
					value,
					false );
    this.curX += KEY_SIZE;
  }
}
